c
c                          demo3.f
c
c     A demo program demonstrating how to produce a simple plot of 
c     3D particle distribution and manipulate it interactively using,
c     in addition to PGPLOT, BUTTON library  written by N.Cardiel and J.Gorgas 
c           (Dept. de Astrofisica, U. Complutense de Madrid)
c
c                 the libary can be downloaded freely from 
c            http://www.ucm.es/info/Astrof/button/button.html
c
c        Andrey Kravtsov ( New Mexico State University, 1998 ) 
c
c     Copyright (c) 1998  Andrey V. Kravtsov. This program can be distributed
c     and used freely for personal, educational or research purposes.  
c     Use for any other purposes requires special arrangements.  
c     
c     Questions and/or comments may be sent to akravtso@nmsu.edu
c
c
      implicit none
c
      integer npmax
      parameter ( npmax = 300000 ) 
      integer ind(npmax)
      real xp(npmax) , yp(npmax) , zp(npmax) , dp(npmax)
c
      integer np , ic1 , id , ierr
      real psu(11) , aexpn, F(8,3)
      integer P
      real SIGN, CONTRA, BRIGHT
      integer ID(3)
      real Phi, Theta, dTh, Zoom
      logical Always
      parameter ( Always = .true. ) 
c
      integer InitDevice, pgslct
      external InitDevice, pgslct
c
c.... read particles
c
      write(*,*) 'reading particles...'
      open ( 10 , file = 'particles.dat' , form = 'unformatted',
     &       status='old')
      read(10) np
      write(*,*) 'np = ', np
      read(10) aexpn
      write(*,*) 'aexpn = ', aexpn
      do ic1 = 1 , np 
        read(10) xp(ic1), yp(ic1), zp(ic1), dp(ic1)
      enddo
      close ( 10 ) 
c
c.... sort particles
c
      write(*,*) 'sorting particles...'
      call SortParticles ( dp , np , ind )
c
c.... Initialize devices
c
      ID(1) = InitDevice ( '/xwin' , 5 )
      ID(2) = InitDevice ( '/gif'  , 4 )
      ID(3) = InitDevice ( '/VCPS' , 5 )
c
c.... set up a view point
c
      Phi   = 0.0
      Theta = 120.0 
      psu(1)  = 0.0    ! x-coordinate (3D) of the box center
      psu(2)  = 0.0    ! y-coordinate (3D) of the box center
      psu(3)  = 0.0    ! z-coordinate (3D) of the box center
      psu(4)  = 0.5    ! scale of the plot for zoom in/out
      psu(9)  = 0.0    ! x-coordinate (3D) of the viewing point
      psu(10) = 0.0    ! y-coordinate (3D) of the viewing point
      psu(11) = 2.5    ! z-coordinate (3D) of the viewing point
c      
c.... set up a palette
c
      P      = 4
      SIGN   = 1.0
      CONTRA = 0.56
      BRIGHT = 0.5      
      Zoom   = 0.2
      dTh    = 10.0
c
c.... XWIN is default device
c
      ierr = pgslct ( 1 ) 
c
c.... Initialize buttons
c
      call InitButtons()
      call PlotButtons()
c
      DO WHILE ( Always )
c
        call SetViewAngle ( Phi, Theta, psu(5), psu(6), psu(7), psu(8) )
c
c....   plot particles
c
        write(*,*) 'plotting particles...'
        call pgbbuf ()
        call pgeras ()      
c
        call PlotParticlesPoints ( xp , yp , zp , dp , ind , np , 
     &                             psu , P , SIGN , CONTRA , BRIGHT )
c
        call InitFrame ( F )
        call DrawFrame ( F , psu )
        call pgebuf
c
        ierr = pgslct ( 1 ) 
        call PlotButtons ()
        call Control ( Phi, Theta, psu , Zoom, dTh, 
     &                 P , SIGN , CONTRA , BRIGHT ) 
c
      ENDDO
c
      END
c
c     --------------------------
      subroutine InitButtons ()
c     --------------------------
c
c     this routine was mimicked from one of the demo's of BUTTON library
c      
      INTEGER MAX_NBUTT  ! maximum number of buttons
      PARAMETER ( MAX_NBUTT = 1000 )
c     maximum number of graphic devices that can be employed simultaneously
      INTEGER MAX_ID_BUTT
      PARAMETER (MAX_ID_BUTT=8)
c
      INTEGER MAX_XBUTT,MAX_YBUTT  ! maximum number of columns and rows
      COMMON/BUTTON1/MAX_XBUTT,MAX_YBUTT
c
      INTEGER PGSCF_BUTT   ! character font in buttons
      COMMON/BUTTON2/PGSCF_BUTT
c
      REAL PGSCH_BUTT  ! font size in buttons
      COMMON/BUTTON3/PGSCH_BUTT
c
      REAL YTEXT_BUTT  ! relative y-position of the text baseline in buttons
      COMMON/BUTTON4/YTEXT_BUTT
c
c     Plot region..: X1VPORT,X2VPORT,Y1VPORT,Y2VPORT
c     Button region: X3VPORT,X4VPORT,Y3VPORT,Y4VPORT
c
      REAL X1VPORT,X2VPORT
      REAL X3VPORT,X4VPORT
      REAL Y1VPORT,Y2VPORT
      REAL Y3VPORT,Y4VPORT
      COMMON/BUTTON5/X1VPORT,X2VPORT,X3VPORT,X4VPORT,Y1VPORT,
     +               Y2VPORT,Y3VPORT,Y4VPORT
c     determines whether the button exist or not
      LOGICAL EXIST_BUTT(MAX_NBUTT)
      COMMON/BUTTON6/EXIST_BUTT
c
c     if .TRUE. tick marks are drawn outside the viewport instead of inside
      LOGICAL ITICKS_BUTT
      COMMON/BUTTON7/ITICKS_BUTT
c
c     if .TRUE. buttons appear in text mode
      LOGICAL MODOTEXT_BUTT
c     if .TRUE. buttons appear in text mode but they are also plotted
      LOGICAL MODOTEXT_PLOTBUTT
      COMMON/BUTTON8/MODOTEXT_BUTT,MODOTEXT_PLOTBUTT
c
      INTEGER TBEGBUTT,TLENBUTT
      INTEGER READIBUTT
      REAL READFBUTT
      CHARACTER*255 READCBUTT
c
      MAX_XBUTT=6
      MAX_YBUTT=2
      PGSCF_BUTT=2
      PGSCH_BUTT=0.7
      YTEXT_BUTT=0.35
      X1VPORT=0.
      X2VPORT=1.
      Y1VPORT=0.0
      Y2VPORT=1.0
      X3VPORT=0.05
      X4VPORT=0.95
      Y3VPORT=0.95
      Y4VPORT=1.
c      
      CALL PGSCR(12,0.5,0.5,0.5)
      CALL PGSCR(13,0.7,0.7,0.7)
      CALL PGSCR(14,0.3,0.3,0.3)
      CALL PGSCR(15,0.6,0.6,0.6)
c
      return
      end
c
c     --------------------------
      subroutine PlotButtons ()
c     --------------------------
c
c     purpose : draw control buttons
c     uses    : button library - see program header
c
      call BUTTON (  1 , 'zoom in'      , 0 )
      call BUTTON (  2 , 'zoom out'     , 0 )
      call BUTTON (  3 , 'rotate left'  , 0 )
      call BUTTON (  4 , 'rotate right' , 0 )
      call BUTTON (  5 , 'fiddle'       , 0 )
      call BUTTON (  6 , 'exit'         , 0 )
      call BUTTON (  7 , 'change zoom'  , 0 )
      call BUTTON (  8 , 'change rot'   , 0 )
      call BUTTON (  9 , 'save as PS'   , 0 )
      call BUTTON ( 10 , 'save as GIF'  , 0 )
c
      return
      end
c
c     ------------------------------------------------
      subroutine Control ( Phi, Theta, psu, Zoom, dTh, 
     &                     P, SIGN, CONTRA, BRIGHT ) 
c     ------------------------------------------------
c
c     implements an interactive control panel via buttons 
c     using BUTTON library 
c
      implicit none
c
      integer IERR, NB
      real XC, YC
      character CH
      character*16 VAL
      real Phi, Theta, dTh, Zoom, psu(11) 
      integer P
      real SIGN, CONTRA, BRIGHT
c
      integer pgslct
      external pgslct
c
 10   call RPGBAND(0,0,0.,0.,XC,YC,CH)
      call IFBUTTON(XC,YC,NB)
c
      if ( NB .eq. 1 ) then 
        call BUTTON(1,'zoom in',5)
        psu(4) = psu(4) + Zoom
        return
      else if ( NB .eq. 2 ) then 
        call BUTTON ( 2 ,'zoom out', 5 )
        psu(4) = psu(4) - Zoom
        if ( psu(4) .lt. 0.0 ) psu(4) = 0.0
        return
      else if ( NB .eq. 3 ) then 
        call BUTTON ( 3 ,'rotate left', 5 )
        Theta = Theta + dTh
        return
      else if ( NB .eq. 4 ) then 
        call BUTTON ( 4 ,'rotate right', 5 )
        Theta = Theta - dTh
        return
      else if ( NB .eq. 5 ) then 
        call BUTTON ( 5 , 'fiddle', 5)
        call FIDDLE ( P , SIGN , CONTRA , BRIGHT )
        call BUTTON ( 5 , 'fiddle', 0 )
        return
      else if ( NB .EQ. 6 ) then
        call BUTTON(6,'exit',5)
        call pgend
        stop
      else if ( NB .eq. 7 ) then 
        call BUTTON ( 7 ,'change zoom', 5 )
        write(*,*) 'Current zoom factor is', Zoom
        write(*,*) 'Enter new value:'
        read(*,*) Zoom
        call BUTTON ( 7 ,'change zoom', 0 )
      else if ( NB .eq. 8 ) then 
        call BUTTON(8,'change rot',5)
        write(*,*) 'Current rotation angle is [deg]',dTh
        write(*,*) 'Enter new value:'
        read(*,*) dTh
        call BUTTON ( 8 ,'change rot', 0 )
      else if ( NB .eq. 9 ) then
        call BUTTON ( 9 ,'save as PS', 5 )
        ierr = pgslct ( 3 )

        return
      else if ( NB .eq. 10 ) then
        call BUTTON ( 10 ,'save as GIF', 5 )
        ierr = pgslct ( 2 )
        return
      endif
       
      goto 10
      end
c
c     ---------------------------------------------
      subroutine SortParticles ( dp , npart , ind )
c     ---------------------------------------------
c
c     Sort particles according to their local density
c     ind(np) has index of particle with largest local density
c
c     input  : npart     - number of particles to sort
c              dp(npart) - density (or any other attribute) to use in sorting
c     output : ind(npart)- particle indices in sorted order
c     
c     uses: indexx subroutine from Numerical Recipes
c
      implicit none
c
      integer npart
      real dp(npart) 
      integer ind(npart)
c
      call indexx ( npart , dp , ind )
c
      return
      end
c
c     ---------------------------------------------------------------
      subroutine PlotParticlesPoints ( xp, yp, zp, dp, ind, npart, 
     &                                 psu, P, SIGN, CONTRA, BRIGHT )
c     ---------------------------------------------------------------
c  
c     Plots input particles coloring them according to the input densities
c
c
      implicit none
c
      integer npart
      integer ind(npart)
      real xp(npart), yp(npart), zp(npart), dp(npart)
      real psu(11)
      integer P
      real CONTRA , BRIGHT , SIGN
c
      integer CI1 , CI2 , ic1 , ip , icolor
      real dmin , dmax , dlmin , dlmax, xpr, ypr
c
      dmin = 1.e7
      dmax = -dmin
      do ic1 = 1 , npart
        if ( dp(ic1) .lt. dmin ) dmin = dp(ic1)
        if ( dp(ic1) .gt. dmax ) dmax = dp(ic1)
      enddo
c
      dlmax = log(dmax)
      if ( dmin .gt. 0.0 ) then 
        dlmin = log(dmin)
      else
        dlmin = -3
      endif
c
      CI1 = 16 
      CI2 = 86
      call PGSCIR ( CI1 , CI2 )         
c
      call Palett ( P , SIGN*CONTRA , BRIGHT )
c
      call PGSITF ( 0 ) 
      call PGSCLP ( 0 )  ! no clipping
c
      do ic1 = 1 , npart
        ip = ind(ic1)
        call OnScreen ( xp(ip) , yp(ip) , zp(ip) , psu , xpr , ypr )
        if ( dp(ip) .gt. 0.0 ) then 
          if ( dp(ip).gt.dmax ) then 
            icolor = 1
          else 
            icolor = int((CI2-CI1)*(log(dp(ip))-dlmin)/(dlmax-dlmin)) 
     &                + CI1
          endif
        else 
          icolor = CI1
        endif
        call pgsci ( icolor )
        call pgpt1 ( xpr , ypr , 1 ) 
      enddo
c
      return
      end
c
c     -----------------------------
      SUBROUTINE indexx(n,arr,indx)
c     -----------------------------
c     (C) Copr. 1986-92 Numerical Recipes Software .-35?421.1-9.
c     ----------------------------------------------------------
      INTEGER n,indx(n),M,NSTACK
      REAL arr(n)
      PARAMETER (M=7,NSTACK=50)
      INTEGER i,indxt,ir,itemp,j,jstack,k,l,istack(NSTACK)
      REAL a
      do 11 j=1,n
        indx(j)=j
11    continue
      jstack=0
      l=1
      ir=n
1     if(ir-l.lt.M)then
        do 13 j=l+1,ir
          indxt=indx(j)
          a=arr(indxt)
          do 12 i=j-1,1,-1
            if(arr(indx(i)).le.a)goto 2
            indx(i+1)=indx(i)
12        continue
          i=0
2         indx(i+1)=indxt
13      continue
        if(jstack.eq.0)return
        ir=istack(jstack)
        l=istack(jstack-1)
        jstack=jstack-2
      else
        k=(l+ir)/2
        itemp=indx(k)
        indx(k)=indx(l+1)
        indx(l+1)=itemp
        if(arr(indx(l+1)).gt.arr(indx(ir)))then
          itemp=indx(l+1)
          indx(l+1)=indx(ir)
          indx(ir)=itemp
        endif
        if(arr(indx(l)).gt.arr(indx(ir)))then
          itemp=indx(l)
          indx(l)=indx(ir)
          indx(ir)=itemp
        endif
        if(arr(indx(l+1)).gt.arr(indx(l)))then
          itemp=indx(l+1)
          indx(l+1)=indx(l)
          indx(l)=itemp
        endif
        i=l+1
        j=ir
        indxt=indx(l)
        a=arr(indxt)
3       continue
          i=i+1
        if(arr(indx(i)).lt.a)goto 3
4       continue
          j=j-1
        if(arr(indx(j)).gt.a)goto 4
        if(j.lt.i)goto 5
        itemp=indx(i)
        indx(i)=indx(j)
        indx(j)=itemp
        goto 3
5       indx(l)=indx(j)
        indx(j)=indxt
        jstack=jstack+2
        if(jstack.gt.NSTACK)pause 'NSTACK too small in indexx'
        if(ir-i+1.ge.j-l)then
          istack(jstack)=ir
          istack(jstack-1)=i
          ir=j-1
        else
          istack(jstack)=j-1
          istack(jstack-1)=l
          l=i
        endif
      endif
      goto 1
      END

